@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2021 S[&]T, The Netherlands.
+ * Copyright (C) 2015-2022 S[&]T, The Netherlands.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -70,6 +70,7 @@ int harp_ingestion_module_geoms_lidar_init(void);
 int harp_ingestion_module_geoms_pandora_init(void);
 int harp_ingestion_module_geoms_sonde_init(void);
 int harp_ingestion_module_geoms_tccon_init(void);
+int harp_ingestion_module_geoms_uvvis_brewer_init(void);
 int harp_ingestion_module_geoms_uvvis_doas_init(void);
 int harp_ingestion_module_geoms_uvvis_dobson_init(void);
 int harp_ingestion_module_gome_l1_init(void);
@@ -135,6 +136,7 @@ static module_init_func_t *module_init_func[] = {
     harp_ingestion_module_geoms_pandora_init,
     harp_ingestion_module_geoms_sonde_init,
     harp_ingestion_module_geoms_tccon_init,
+    harp_ingestion_module_geoms_uvvis_brewer_init,
     harp_ingestion_module_geoms_uvvis_doas_init,
     harp_ingestion_module_geoms_uvvis_dobson_init,
     harp_ingestion_module_gome_l1_init,
